@@ -1,7 +1,6 @@
 """ Dimarray module
     ===============
 """
-from __future__ import absolute_import
 
 import warnings
 from . import core
@@ -14,13 +13,7 @@ from .config import set_option, get_option, print_options, rcParams, rcParamsHel
 from .core import *
 from .dataset import Dataset, stack_ds, concatenate_ds
 from .datasets import get_datadir, get_ncfile
-#from lib.transform import interp1d, interp2d, apply_recursive
 
-# need to fix that
-try:
-    from dimarray.version import version as __version__
-except:
-    __version__ = "X.X.X"
 
 try:
     from .io.nc import (read_nc, open_nc, summary_nc,
@@ -30,5 +23,9 @@ try:
 except ImportError:
     _ncio = False
     msg = "Could not import netCDF4's package ==> I/O will not be available in this format"
-    print msg
+    print(msg)
     #warnings.warn(ImportWarning(msg)) 
+
+
+from pathlib import Path
+__version__ = open(Path(__file__).parent / "_version.py").read()
