@@ -1,10 +1,3 @@
-#from transform import *
-#from reshape import *
-#from indexing import *
-#from interpolation import *
-#from missingvalues import *
-from __future__ import absolute_import
-
 from dimarray.tools import pandas_obj, deprecated_func
 from .dimarraycls import DimArray, array, empty, zeros, ones, nans, empty_like, zeros_like, ones_like, nans_like, from_pandas, from_arrays
 from .axes import Axis, Axes, MultiAxis
