@@ -1,6 +1,5 @@
 """ Functions and dimarray methods associated to array alignment
 """
-from collections import OrderedDict as odict
 import itertools
 import warnings
 import numpy as np
@@ -13,14 +12,14 @@ from dimarray.core.indexing import locate_many
 __all__ = ["broadcast_arrays", "align", "stack", "concatenate"]
 
 def get_dims(*arrays):
-    """ find all dimensions from a variable list of arrays
+    """ find all dimensions from a variable list of arrays (or any object with `axes` attribute)
     Note: not in public API, but used by other modules
     """
     dims = []
     for o in arrays:
-        for dim in o.dims:
-            if dim not in dims:
-                dims.append(dim)
+        for ax in o.axes:
+            if ax.name not in dims:
+                dims.append(ax.name)
     return dims
 
 def _get_axes(*arrays):
@@ -133,8 +132,8 @@ def broadcast_arrays(*arrays):
         axes = _get_axes(*arrays)
 
     # fails if axes are not aligned
-    except AssertionError, msg:
-        raise ValueError(msg)
+    except AssertionError as error:
+        raise ValueError(error)
 
     # now broadcast each DimArray along commmon axes
     newarrays = []
@@ -175,38 +174,81 @@ def _common_axis(axes, join):
         com_axis = ax0.intersection(ax1)
     return com_axis
 
-def align(*arrays, **kwargs):
+def _get_aligned_axes(arrays, join='outer', axis=None , sort=False, strict=False):
+    """From a list of arrays, or any object with `axes` attributes, 
+    a new list of axes.
+    """
+    # find the dimensions
+    if axis is None:
+        dims = get_dims(*arrays)
+    elif isinstance(axis, str):
+        dims = [axis]
+    else:
+        if not isinstance(axis, str):
+            raise ValueError("align: axis must be provided as a string")
+
+    axes = Axes()
+
+    for jj, d in enumerate(dims):
+
+        # arrays which have that dimension
+        ii = [i for i in range(len(arrays)) if d in arrays[i].dims]
+
+        if strict and len(ii) != len(arrays):
+            raise ValueError("align (strict=True): some arrays lack dimension {}".format(d))
+
+        # common axis to reindex on
+        ax = _common_axis([arrays[i].axes[d] for i in ii], join)
+
+        if sort:
+            ax.sort()
+
+        axes.append(ax)
+
+    # assert len(axes) > 0
+
+    return axes
+
+def align(arrays, join='outer', axis=None , sort=False, strict=False):
     """Align axes of a list of DimArray arrays by reindexing
 
     Parameters
     ----------
-    array1, array2, ... : variable list of DimArrays
+    array1, array2, ... : variable list of DimArrays or Datasets
     join : {"outer", "inner"}, optional
         method to find the common axis
         "outer" : union of all axes, missing values filled with NaNs
         "inner" : intersection of all axes
         Default to "outer" (can be changed with `dimarray.set_option('align.join","inner")`)
+    sort : bool, optional
+        Sort the axis prior to aligning.
+        default to False
+    axis : str, optional
+        default to None : align all axes
+        (must be a string since the axes do not necessarily match)
+    strict : bool, optional
+        if True, check that all arrays have the same dimensions
 
     Returns
     -------
-    aligned_array1, aligned_array2, ... : list of aligned DimArrays
+    aligned_array1, aligned_array2, ... : list of aligned DimArrays (or Dataset)
 
     See Also
     --------
-    `DimArray.reindex`, `DimArray.reindex_like`
+    `DimArray.reindex_axis`, `DimArray.reindex_like`
 
     Examples
     --------
     >>> from dimarray import DimArray, align
     >>> a = DimArray([0,1,2],axes=[[0,1,2]])
     >>> b = DimArray([1,2,3],axes=[[1,2,3]])
-    >>> align(a, b)
+    >>> align([a, b])
     [dimarray: 3 non-null elements (1 null)
     0 / x0 (4): 0 to 3
-    array([  0.,   1.,   2.,  nan]), dimarray: 3 non-null elements (1 null)
+    array([ 0.,  1.,  2., nan]), dimarray: 3 non-null elements (1 null)
     0 / x0 (4): 0 to 3
-    array([ nan,   1.,   2.,   3.])]
-    >>> align(a, b, join='inner')
+    array([nan,  1.,  2.,  3.])]
+    >>> align([a, b], join='inner')
     [dimarray: 2 non-null elements (0 null)
     0 / x0 (2): 1 to 2
     array([1, 2]), dimarray: 2 non-null elements (0 null)
@@ -217,53 +259,50 @@ def align(*arrays, **kwargs):
      
     >>> a = DimArray([0,1], axes=[[0,1]]) # on 'x0' only
     >>> b = DimArray([[0,1],[2,3.],[4.,5.]], axes=[[0,1,2],[1,2]]) # one more element along the 1st dimension, 2nd dimension ignored
-    >>> align(a, b)
+    >>> align([a, b])
     [dimarray: 2 non-null elements (1 null)
     0 / x0 (3): 0 to 2
-    array([  0.,   1.,  nan]), dimarray: 6 non-null elements (0 null)
+    array([ 0.,  1., nan]), dimarray: 6 non-null elements (0 null)
     0 / x0 (3): 0 to 2
     1 / x1 (2): 1 to 2
-    array([[ 0.,  1.],
-           [ 2.,  3.],
-           [ 4.,  5.]])]
+    array([[0., 1.],
+           [2., 3.],
+           [4., 5.]])]
     """
-    join = kwargs.pop('join', get_option('align.join'))
-    if len(kwargs) > 0:
-        raise TypeError("align() got unexpected argument(s): "+", ".join(kwargs.keys()))
+    # join = kwargs.pop('join', get_option('align.join'))
+    # sort = kwargs.pop('sort', False)
+    # axis = kwargs.pop('axis', None)
+    # strict = kwargs.pop('strict', False)
+    # if len(kwargs) > 0:
+    #     raise TypeError("align() got unexpected argument(s): "+", ".join(kwargs.keys()))
+    if not (isinstance(arrays, list) or isinstance(arrays, tuple)):
+        raise ValueError("align: only accepts list or tuple arguments. Got: {}".format(type(arrays)))
 
     # convert any scalar to dimarray
-    from dimarray import DimArray
-    arrays = list(arrays)
+    from dimarray import DimArray, Dataset
+    arrays = [a for a in arrays] # convert to list
     for i, a in enumerate(arrays):
-        if not isinstance(a, DimArray):
+        if not isinstance(a, DimArray) and not isinstance(a, Dataset):
             if np.isscalar(a):
                 arrays[i] = DimArray(a)
             else:
-                raise TypeError("can only align DimArray instances")
+                raise TypeError("can only align DimArray and Dataset instances, got: {}".format(type(a)))
 
-    # find the dimensiosn
-    dims = get_dims(*arrays)
+    # find the common axes
+    axes = _get_aligned_axes(arrays, axis=axis, join=join, sort=sort, strict=strict)
 
-    for jj, d in enumerate(dims):
-
-        # arrays which have that dimension
-        ii = filter(lambda i: d in arrays[i].dims, range(len(arrays)))
-
-        # common axis to reindex on
-        ax_values = _common_axis([arrays[i].axes[d] for i in ii], join)
-
-        # update arrays
+    # update arrays
+    for ax in axes:
         for i, o in enumerate(arrays):
-            if i not in ii:
+            if ax.name not in o.dims: 
                 continue
-            if np.all(o.axes[d] == ax_values):
+            if np.all(o.axes[ax.name] == ax):
                 continue
-
-            arrays[i] = o.reindex_axis(ax_values, axis=d)
+            arrays[i] = o.reindex_axis(ax)
 
     return arrays
 
-align_axes = align  # for internal use, so that it does not conflict with "align" parameter
+align_ = align  # for internal use, so that it does not conflict with "align" parameter
 
 def _check_stack_args(arrays, keys=None):
     """ generic function to deal with arguments for stacking
@@ -296,15 +335,15 @@ def _check_stack_axis(axis, dims, default='unnamed'):
                 i+=1
             axis = default+"_{}".format(i)
 
-    elif type(axis) is int:
+    if type(axis) is int:
         raise TypeError("axis must be a str (new axis name)")
 
     if axis in dims:
         raise ValueError("please provide an axis name which does not \
-                already exist")
+                already exist, or use `concatenate`")
     return axis
 
-def stack(arrays, axis=None, keys=None, align=False):
+def stack(arrays, axis=None, keys=None, align=False, **kwargs):
     """ stack arrays along a new dimension (raise error if already existing)
 
     Parameters
@@ -316,6 +355,7 @@ def stack(arrays, axis=None, keys=None, align=False):
         stack axis values, useful if array is a sequence, or a non-ordered dictionary
     align : bool, optional
         if True, align axes prior to stacking (Default to False)
+    **kwargs : optional key-word arguments passed to align, if align is True
 
     Returns
     -------
@@ -352,7 +392,8 @@ def stack(arrays, axis=None, keys=None, align=False):
 
     # re-index axes if needed
     if align:
-        arrays = align_axes(*arrays)
+        kwargs['strict'] = True
+        arrays = align_(arrays, **kwargs)
 
     # make it a numpy array
     data = [a.values for a in arrays]
@@ -364,7 +405,7 @@ def stack(arrays, axis=None, keys=None, align=False):
     # find common axes
     try: 
         axes = _get_axes(*arrays)
-    except ValueError, msg: 
+    except ValueError as msg: 
         if 'axes are not aligned' in repr(msg):
             msg = 'axes are not aligned\n ==> Try passing `align=True`' 
         raise ValueError(msg)
@@ -393,12 +434,12 @@ def _concatenate_axes(axes):
     #assert np.iterable(axes) and axes
     #if not isinstance(axes[0], Axis): raise TypeError()
     if len({ax.name for ax in axes}) != 1: 
-        print axes
+        print(axes)
         raise ValueError("axis names differ!")
     values = np.concatenate([ax.values for ax in axes])
     return Axis(values, axes[0].name)
 
-def concatenate(arrays, axis=0, check_other_axes=True):
+def concatenate(arrays, axis=0, _no_check=False, align=False, **kwargs):
     """ concatenate several DimArrays
 
     Parameters
@@ -406,7 +447,11 @@ def concatenate(arrays, axis=0, check_other_axes=True):
     arrays : list of DimArrays
         arrays to concatenate
     axis : int or str 
-        axis along which to concatenate
+        axis along which to concatenate (must exist)
+    align : bool, optional
+        align secondary axes before joining on the primary
+        axis `axis`. Default to False.
+    **kwargs : optional key-word arguments passed to align, if align is True
 
     Returns
     -------
@@ -415,6 +460,7 @@ def concatenate(arrays, axis=0, check_other_axes=True):
     See Also
     --------
     stack: join arrays along a new dimension
+    align: align arrays
 
     Examples
     --------
@@ -448,29 +494,57 @@ def concatenate(arrays, axis=0, check_other_axes=True):
     array([[ 1,  2,  3,  4,  5,  6],
            [11, 22, 33, 44, 55, 66]])
     """
-    assert type(arrays) in (list, tuple), "arrays must be list or tuple, got {}:{}".format(type(arrays), arrays)
+    # input argument check
+    if not type(arrays) in (list, tuple):
+        raise ValueError("arrays must be list or tuple, got {}:{}".format(type(arrays), arrays))
+    arrays = [a for a in arrays]
+
+    from dimarray import DimArray, Dataset
+
+    for i, a in enumerate(arrays):
+        if isinstance(a, Dataset):
+            msg = "\n==>Note: you may use `concatenate_ds` for Datasets"
+            raise ValueError("concatenate: expected DimArray. Got {}".format(type(a))+msg)
+        elif np.isscalar(a):
+            arrays[i] = DimArray(a)
+        if not isinstance(a, DimArray):
+            raise ValueError("concatenate: expected DimArray. Got {}".format(type(a)))
 
     if type(axis) is not int:
         axis = arrays[0].dims.index(axis)
+    dim = arrays[0].dims[axis]
+
+    # align secondary axes prior to concatenate
+    # TODO: just encourage user to use align outside this function
+    # and remove argument passing
+    if align:
+        kwargs['strict'] = True
+        for ax in arrays[0].axes:
+            if ax.name != dim:
+                arrays = align_(arrays, axis=ax.name, **kwargs)
 
     values = np.concatenate([a.values for a in arrays], axis=axis)
 
-    _get_subaxes = lambda x: [ax for i, ax in enumerate(a.axes) if i != axis]
+    _get_subaxes = lambda x: [ax for i, ax in enumerate(arrays[0].axes) if i != axis]
     subaxes = _get_subaxes(arrays[0])
-    #assert np.all(_get_subaxes(a) == subaxes for a in arrays),"some axes do not match"
 
     # concatenate axis values
-    #newaxisvalues = np.concatenate([a.axes[axis].values for a in arrays])
-    #newaxis = Axis(newaxisvalues, name=arrays[0].dims[axis])
     newaxis = _concatenate_axes([a.axes[axis] for a in arrays])
 
-    # check that other axes match
-    if check_other_axes:
-        for i,a in enumerate(arrays[1:]):
-            if not _get_subaxes(a) == subaxes:
-                msg = "First array:\n{}\n".format(subaxes)
-                msg += "{}th array:\n{}\n".format(i,_get_subaxes(a))
-                raise ValueError("contatenate: other axes to not match.")
+    if not align and not _no_check:
+        # check that other axes match
+        for ax in subaxes:
+            for a in arrays:
+                if not np.all(a.axes[ax.name].values == ax.values):
+                    raise ValueError("contatenate: secondary axes do not match. Align first? (`align=True`)")
+        # print arrays[0]
+        # for i,a in enumerate(arrays[1:]):
+        #     if not _get_subaxes(a) == subaxes:
+        #         msg = "First array:\n{}\n".format(subaxes)
+        #         msg += "{}th array:\n{}\n".format(i,_get_subaxes(a))
+        #         raise ValueError("contatenate: secondary axes do not match. Align first? (`align=True`)")
+        #     print a
+        # print '==> arrays look ok'
 
     newaxes = subaxes[:axis] + [newaxis] + subaxes[axis:]
 
@@ -516,7 +590,7 @@ def reindex_axis(self, values, axis=0, fill_value=np.nan, raise_error=False, met
     >>> b.reindex_axis([1,2,3])
     dimarray: 2 non-null elements (1 null)
     0 / x0 (3): 1 to 3
-    array([  3.,  nan,   4.])
+    array([ 3., nan,  4.])
 
     Or replace with anything else, like -9999
 
@@ -600,7 +674,7 @@ def reindex_like(self, other, **kwargs):
     >>> b.reindex_like(c)
     dimarray: 2 non-null elements (1 null)
     0 / x0 (3): 1 to 3
-    array([  3.,  nan,   4.])
+    array([ 3., nan,  4.])
     """
     if hasattr(other, 'axes'):
         axes = other.axes
