@@ -1,11 +1,9 @@
-from __future__ import absolute_import, division
 import warnings
-from collections import OrderedDict as odict
 import string
 import copy
 import numpy as np
-
-from dimarray.tools import is_DimArray, is_array1d_equiv, format_doc
+from dimarray.compat.pycompat import zip
+from dimarray.tools import is_DimArray, is_array1d_equiv, format_doc, isscalar
 from dimarray.core.bases import AbstractAxis, AbstractAxes, GetSetDelAttrMixin
 from dimarray.core.indexing import _maybe_cast_type, is_monotonic
 
@@ -40,7 +38,7 @@ def _check_axis_values(values, dtype=None):
     if values.ndim == 2: 
         try:
             val = np.empty(values.shape[0], dtype=object)
-            val[:] = zip(*values.T.tolist()) # pass a list of tuples
+            val[:] = list(zip(*values.T.tolist())) # pass a list of tuples
             values = val
         except:
             pass
@@ -96,18 +94,16 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
     values : numpy array (or list) 
     name : name (attribute)
 
-    weights : [None] associated list of weights 
     tol : [None], if not None, attempt a nearest neighbour search with specified tolerance
     """
-    __metadata_exclude__ = ['values','name','weights']
+    __metadata_exclude__ = ['values','name']
 
-    def __init__(self, values, name="", weights=None, dtype=None, tol=None, **kwargs):
+    def __init__(self, values, name="", dtype=None, tol=None, **kwargs):
         self.name = name or getattr(values, "name", "")
         self._values = _check_axis_values(values, dtype)
         self.name = name 
-        self.weights = weights # additional checks
         self._tol = tol
-        self._attrs = odict()
+        self._attrs = dict()
         self._attrs.update(kwargs)
         self._monotonic = None
 
@@ -130,7 +126,7 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
 
     @name.setter
     def name(self, name):
-        if not isinstance(name, basestring):
+        if not isinstance(name, str):
             raise TypeError("Axis name must be a string")
         if not name:
             raise ValueError("Axis name cannot be empty")
@@ -143,42 +139,23 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
     def tol(self, val):
         self._tol = val
 
-    @property
-    def weights(self):
-        return self._weights
 
-    @weights.setter
-    def weights(self, _weights):
-        if _weights is None or callable(_weights):
-            pass
-        else:
-            try:
-                _weights = np.asarray(_weights)
-            except:
-                raise TypeError("weight must be array-like or callable, got: {}".format(_weights))
-            if _weights.size != self.size:
-                raise ValueError("weights must have the same size as axis values, got: {} and {} !".format(_weights.size, self.size))
-        self._weights = _weights
-
-    @weights.deleter
-    def weights(self):
-        self._weights = None
+    def sort(self, *args, **kwargs):
+        # in-place sort
+        self._values.sort(*args, **kwargs)
+        self._monotonic = True
 
     def __getitem__(self, item):
         """ access values elements & return an axis object
         """
         if type(item) is slice and item == slice(None):
             return self
-        if not isinstance(item, slice) and not np.isscalar(item):
+        if not isinstance(item, slice) and not isscalar(item):
             item = np.asarray(item) # needed for boolean axes, otherwise problem
         values = self.values[item]
         if not isinstance(values, np.ndarray):
             return values # if collapsed to scalar, just return it
-        if isinstance(self.weights, np.ndarray):
-            weights = self.weights[item]
-        else:
-            weights = self.weights
-        newaxis = Axis(values, self.name, weights=weights, tol=self.tol, **self.attrs)
+        newaxis = Axis(values, self.name, tol=self.tol, **self.attrs)
         # slices keep the ordering
         if self._monotonic and type(item) is slice:
             newaxis._monotonic = self._monotonic
@@ -194,7 +171,7 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
         array([1, 2, 3])
         >>> a[0] = 1.2  # convert to float
         >>> a.values
-        array([ 1.2,  2. ,  3. ])
+        array([1.2, 2. , 3. ])
         >>> a[0] = 'a'  # convert to object dtype
         >>> a.values
         array(['a', 2.0, 3.0], dtype=object)
@@ -224,11 +201,7 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
         subaxis : Axis instance
         """
         values = self._values.take(indices, mode=mode)
-        if isinstance(self.weights, np.ndarray):
-            weights = self.weights.take(indices, mode=mode)
-        else:
-            weights = self.weights
-        return Axis(values, self.name, weights=weights, tol=self.tol, **self.attrs)
+        return Axis(values, self.name, tol=self.tol, **self.attrs)
 
 
     def set(self, values=None, name=None, inplace=True, **kwargs):
@@ -251,7 +224,7 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
         **kwargs : key-word arguments
             Also reset other axis attributes, which can be single metadata
             or other axis attributes, via using `setattr`
-            This includes special attributes `weights` and `attrs` (the latter
+            This includes special attributes `attrs` (the latter
             reset all attributes)
 
         Returns
@@ -313,8 +286,8 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
         #assert isinstance(other, Axis), "can only make the Union of two Axis objects"
         self, other, consistent_kinds = _check_axes_merge(self, other)
         
-        if np.all(self.values == other.values):
-            # TODO: check other attributes such as weights
+        if self.values.size == other.values.size and np.all(self.values == other.values):
+            # TODO: check other attributes 
             return self.copy()
         elif self.values.size == 0:
             return other
@@ -345,8 +318,8 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
         """
         self, other, consistent_kinds = _check_axes_merge(self, other)
 
-        if np.all(self.values == other.values):
-            # TODO: check other attributes such as weights
+        if self.values.shape == other.values.shape and np.all(self.values == other.values):
+            # TODO: check other attributes 
             return self.copy()
         elif self.values.size == 0:
             return Axis([], self.name)
@@ -375,7 +348,7 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
 
     def __eq__(self, other):
         #return hasattr(other, "name") and hasattr(other, "values") and np.all(other.values == self.values) and self.name == other.name
-        return isinstance(other, Axis) and np.all(other.values == self.values) and self.name == other.name
+        return isinstance(other, Axis) and other.size == self.values.size and np.all(other.values == self.values) and self.name == other.name
 
     def _bounds(self):
         if self.values.size == 0:
@@ -403,41 +376,6 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
     def __array__(self): 
         return self.values.__array__
 
-    def _get_weights(self, weights=None):
-        """ return axis weights as a DimArray
-
-        Parameters
-        ----------
-        weights : array-like or callable, optional
-            if provided, will be used instead of self.weights
-            used in weighted transformations (see doc in there)
-        """
-        from .dimarraycls import DimArray
-
-        if weights is None:
-            weights = self.weights
-
-        # no weights
-        if weights is None:
-            weights = np.ones_like(self.values)
-
-        # function of axis
-        elif callable(weights):
-            weights = weights(self.values)
-
-        # same weight for every element
-        elif np.size(weights) == 1:
-            weights = np.zeros_like(self.values) + weights
-
-        # already an array of weights
-        else:
-            weights = np.asarray(weights)
-
-        # index on one dimension
-        ax = Axis(self.values, name=self.name)
-
-        return DimArray(weights, [ax])
-
     def to_pandas(self):
         """ convert to pandas Index
         """
@@ -460,7 +398,7 @@ class Axis(GetSetDelAttrMixin, AbstractAxis):
             try:
                 ax = cls(np.asarray(ax), getattr(ax,'name',defaultname))
             except Exception as error: 
-                print error.message
+                print(error.message)
                 raise TypeError("Cannot convert to Axis object: {}. \nPlease provide an Axis instance or (name, values) tuple".format(type(ax)))
         return ax
 
@@ -476,9 +414,8 @@ class MultiAxis(Axis):
         self.axes = Axes(axes)
         self._name = ",".join([ax.name for ax in self.axes])
         self._values = None  # values not computed unless needed
-        self._weights = None  
         self._size = None  
-        self._attrs = odict()
+        self._attrs = dict()
 
     @property
     def values(self):
@@ -495,21 +432,9 @@ class MultiAxis(Axis):
 
         aval = _flatten(*[ax.values for ax in self.axes])
         val = np.empty(aval.shape[0], dtype=object)
-        val[:] = zip(*aval.T.tolist()) # pass a list of tuples
+        val[:] = list(zip(*aval.T.tolist())) # pass a list of tuples
         return val 
 
-    @property
-    def weights(self):
-        """ Combine the weights as a product of the individual axis weights
-        """
-        if self._weights is None:
-            #self._weights = self._get_weights()
-            if np.all([ax.weights is None for ax in self.axes]):
-                self._weights = None
-            else:
-                self._weights =_flatten(*[ax._get_weights() for ax in self.axes]).prod(axis=1)
-        return self._weights
-
     @property
     def size(self): 
         """ size as product of axis sizes
@@ -555,6 +480,9 @@ class MultiAxis(Axis):
             ax.name = mi.name
         return ax
 
+    def sort(self, *args, **kwargs):
+        raise NotImplementedError()
+
 def _flatten(*list_of_arrays):
     """ flatten a list of arrays ax1, ax2, ... to  a list of tuples [(ax1[0], ax2[0], ax3[0]..), (ax1[0], ax2[0], ax3[1]..), ...]
     """
@@ -563,7 +491,7 @@ def _flatten(*list_of_arrays):
         return list_of_arrays[0]
     kwargs = dict(indexing="ij")
     grd = np.meshgrid(*list_of_arrays, **kwargs)
-    array_of_tuples = np.array(zip(*[g.ravel() for g in grd]))
+    array_of_tuples = np.array(list(zip(*[g.ravel() for g in grd])))
     assert array_of_tuples.shape[1] == len(list_of_arrays), "pb when reshaping: {} and {}".format(array_of_tuples.shape, len(list_of_arrays))
     assert array_of_tuples.shape[0] == np.prod([x.size for x in list_of_arrays]), "pb when reshaping: {} and {}".format(array_of_tuples.shape, np.prod([x.size for x in list_of_arrays]))
     return array_of_tuples
@@ -613,7 +541,7 @@ class Axes(AbstractAxes, list):
         if dims is None: 
             dims = ["x{}".format(i) for i in range(len(arrays))]
 
-        return cls(zip(dims, arrays))
+        return cls(list(zip(dims, arrays)))
 
     @classmethod
     def from_dict(cls, kwaxes, dims=None, shape=None, check_order=True):
@@ -671,7 +599,7 @@ class Axes(AbstractAxes, list):
 
     def __getitem__(self, k):
         " get an axis by integer or name "
-        if isinstance(k, basestring):
+        if isinstance(k, str):
             dims = [ax.name for ax in self]
             try:
                 k = dims.index(k)
@@ -683,7 +611,7 @@ class Axes(AbstractAxes, list):
     def __setitem__(self, k, newax):
         """ update existing axis, the size cannot be changed
         """
-        if isinstance(k, basestring):
+        if isinstance(k, str):
             k = [ax.name for ax in self].index(k)
         curax = list.__getitem__(self, k)
 
@@ -722,7 +650,7 @@ class Axes(AbstractAxes, list):
 
     def _get_idx(self, axis):
         " always return axis integer location "
-        if isinstance(axis, basestring):
+        if isinstance(axis, str):
             dims = [ax.name for ax in self]
             try:
                 axis = dims.index(axis)
@@ -777,7 +705,7 @@ def _init_axes(axes=None, dims=None, labels=None, shape=None, check_order=True):
 
     elif isinstance(axes, dict):
         kwaxes = axes
-        if isinstance(kwaxes, odict) and dims is None:
+        if isinstance(kwaxes, dict) and dims is None:
             dims = kwaxes.keys()
         axes = Axes.from_dict(kwaxes, dims=dims, shape=shape, check_order=check_order)
         return axes
@@ -801,11 +729,11 @@ def _init_axes(axes=None, dims=None, labels=None, shape=None, check_order=True):
         axes = Axes(axes)
 
     # axes contains only axis values, with names possibly provided in `dims=`
-    elif np.all([type(ax) in (list, np.ndarray) for ax in axes]):
+    elif np.all([type(ax) in (list, np.ndarray, np.ma.MaskedArray) for ax in axes]):
         axes = Axes.from_arrays(axes, dims=dims)
 
     # axes only cointain axis labels
-    elif np.all([type(ax) in (str, unicode) for ax in axes]):
+    elif np.all([isinstance(ax,str) for ax in axes]):
         axes = Axes.from_shape(shape, dims=axes)
 
     else:
