@@ -1,15 +1,12 @@
 """ Base classes
 """
-from __future__ import absolute_import
 import warnings
 import copy
-from collections import OrderedDict as odict
 import numpy as np
 from dimarray.config import get_option
-from dimarray.compat.pycompat import iteritems, zip
 from dimarray.tools import is_numeric
-from .indexing import locate_one, locate_many, locate_slice, expanded_indexer
-from .prettyprinting import repr_axis, repr_dataset, repr_axes, str_axes, str_dataset, str_dimarray
+from dimarray.core.indexing import locate_one, locate_many, locate_slice, expanded_indexer
+from dimarray.prettyprinting import repr_axis, repr_dataset, repr_axes, str_axes, str_dataset, str_dimarray
 
 class GetSetDelAttrMixin(object):
     """ Class to overload the __getattr__, __setattr__, __delattr__
@@ -26,7 +23,7 @@ class GetSetDelAttrMixin(object):
         elif name not in self.__metadata_include__ \
                 and (name.startswith('_') or name in self.__metadata_exclude__):
             pass # raise error
-        elif hasattr(self, 'dims') and name in self.dims:
+        elif hasattr(type(self), 'dims') and name in self.dims:
             return self.axes[name].values # return axis values
         elif name in self.attrs.keys():
             return self.attrs[name]
@@ -38,7 +35,7 @@ class GetSetDelAttrMixin(object):
                  or name in self.__metadata_exclude__ \
                  or hasattr(self.__class__, name)):
             object.__setattr__(self, name, value) # do nothing special
-        elif hasattr(self, 'axes') and name in self.dims:
+        elif hasattr(type(self), 'axes') and name in self.dims:
             self.axes[name][()] = value # modify axis values
         else:
             self.attrs[name] = value # add as metadata
@@ -52,6 +49,8 @@ class GetSetDelAttrMixin(object):
         else:
             return object.__delattr__(self, name)
 
+
+
 # class GetSetDelAttrMixin(object):
 #     pass
 #
@@ -68,7 +67,7 @@ class AbstractHasMetadata(object):
          
     @attrs.deleter
     def attrs(self):
-        for k in self.attrs.keys():
+        for k in list(self.attrs.keys()):
             del self.attrs[k]
 
     def _repr(self, metadata=False):
@@ -80,7 +79,7 @@ class AbstractHasMetadata(object):
     __str__ = str_dimarray
 
     def summary(self):
-        print self.summary_repr()
+        print(self.summary_repr())
 
     def summary_repr(self):
         return self._repr(metadata=True)
@@ -307,8 +306,8 @@ class AbstractHasAxes(AbstractHasMetadata):
         # should always be a tuple
         if isinstance(indices, dict):
             # replace int dimensions with str dimensions
-            for k in indices:
-                if not isinstance(k, basestring):
+            for k in list(indices):
+                if not isinstance(k, str):
                     indices[dims[k]] = indices[k]
                     del indices[k] 
                 else:
@@ -373,7 +372,7 @@ class AbstractHasAxes(AbstractHasMetadata):
         if axis is None:
             return None, None
 
-        if type(axis) in (str, unicode):
+        if isinstance(axis, str):
             idx = self.dims.index(axis)
 
         elif type(axis) is int:
@@ -485,6 +484,9 @@ class AbstractDimArray(AbstractHasAxes):
     @property
     def size(self):
         return self.values.size
+    @property
+    def item(self):
+        return self.values.item
 
     def is_numeric(self):
         return is_numeric(self.values)
@@ -530,6 +532,7 @@ class AbstractDimArray(AbstractHasAxes):
         if np.isscalar(values):
             return values
 
+
         dima = self._constructor(values, axes) # initialize DimArray
         dima.attrs.update(self.attrs) # add attribute
 
@@ -610,8 +613,6 @@ class AbstractDataset(AbstractHasAxes):
         newdata = self.__class__()
 
         # then apply take in 'position' mode
-        newdata = self.__class__()
-
         axes_dict = {ax.name:ax[ix] for ix, ax in zip(tuple_indices, self.axes) if not np.isscalar(ix)}
         indices_dict = {ax.name:ix for ix, ax in zip(tuple_indices, self.axes)}
 
