@@ -12,7 +12,7 @@ except ImportError:
     _hasbottleneck = False
 
 from dimarray.tools import anynan, is_DimArray, format_doc
-from axes import Axis, Axes
+from dimarray.core.axes import Axis, Axes
 
 
 #
@@ -108,29 +108,29 @@ def apply_along_axis(self, func, axis=None, skipna=False, args=(), **kwargs):
     0 / items (2): 'a' to 'b'
     1 / x0 (2): 0 to 1
     2 / x1 (2): 0 to 1
-    array([[[  0.,   1.],
-            [  2.,   3.]],
+    array([[[ 0.,  1.],
+            [ 2.,  3.]],
     <BLANKLINE>
-           [[ nan,   1.],
-            [  2.,   3.]]])
+           [[nan,  1.],
+            [ 2.,  3.]]])
     >>> c.sum(axis=0)
     dimarray: 3 non-null elements (1 null)
     0 / x0 (2): 0 to 1
     1 / x1 (2): 0 to 1
-    array([[ nan,   2.],
-           [  4.,   6.]])
+    array([[nan,  2.],
+           [ 4.,  6.]])
     >>> c.sum(0, skipna=True)
     dimarray: 4 non-null elements (0 null)
     0 / x0 (2): 0 to 1
     1 / x1 (2): 0 to 1
-    array([[ 0.,  2.],
-           [ 4.,  6.]])
+    array([[0., 2.],
+           [4., 6.]])
     >>> c.median(0)
     dimarray: 3 non-null elements (1 null)
     0 / x0 (2): 0 to 1
     1 / x1 (2): 0 to 1
-    array([[ nan,   1.],
-           [  2.,   3.]])
+    array([[nan,  1.],
+           [ 2.,  3.]])
     """
 
 
@@ -350,9 +350,9 @@ prod = _NumpyDesc("prod")
 sum  = _NumpyDesc("sum")
 
 # here just for checking
-_mean = _NumpyDesc("mean")
-_std = _NumpyDesc("std")
-_var = _NumpyDesc("var")
+mean = _NumpyDesc("mean")
+std = _NumpyDesc("std")
+var = _NumpyDesc("var")
 
 min = _NumpyDesc("min")
 max = _NumpyDesc("max")
@@ -471,35 +471,35 @@ def diff(self, axis=-1, scheme="backward", keepaxis=False, n=1):
     >>> s 
     dimarray: 4 non-null elements (0 null)
     0 / time (4): 1950 to 1953
-    array([  1.,   3.,   6.,  10.])
+    array([ 1.,  3.,  6., 10.])
 
     `diff` reduces axis size by one, by default
 
     >>> s.diff()
     dimarray: 3 non-null elements (0 null)
     0 / time (3): 1951 to 1953
-    array([ 2.,  3.,  4.])
+    array([2., 3., 4.])
 
     The `keepaxis=` parameter fills array with `nan` where necessary to keep the axis unchanged. Default is backward differencing: `diff[i] = v[i] - v[i-1]`.
 
     >>> s.diff(keepaxis=True)
     dimarray: 3 non-null elements (1 null)
     0 / time (4): 1950 to 1953
-    array([ nan,   2.,   3.,   4.])
+    array([nan,  2.,  3.,  4.])
 
     But other schemes are available to control how the new axis is defined: `backward` (default), `forward` and even `centered`
 
     >>> s.diff(keepaxis=True, scheme="forward") # diff[i] = v[i+1] - v[i]
     dimarray: 3 non-null elements (1 null)
     0 / time (4): 1950 to 1953
-    array([  2.,   3.,   4.,  nan])
+    array([ 2.,  3.,  4., nan])
 
     The `keepaxis=True` option is invalid with the `centered` scheme, since every axis value is modified by definition:
 
     >>> s.diff(axis='time', scheme='centered')
     dimarray: 3 non-null elements (0 null)
     0 / time (3): 1950.5 to 1952.5
-    array([ 2.,  3.,  4.])
+    array([2., 3., 4.])
     """
     # If `axis` is None (operations on the flattened array), just returns the numpy array
     if axis is None:
@@ -603,377 +603,6 @@ def _append_nans(result, axis, first=False):
 #        return obj.axes[idx].values[result] 
 
 
-
-#
-# Define weighted mean/std/var
-#
-_doc_weights = dict(
-
-        axis = """
-    axis : int or str or tuple, optional
-        axis or sequence of axes to apply the transform on
-        """.strip(),
-
-        skipna = """
-    skipna : bool, optional
-        ignore missing values (nans) prior to transformation
-        Default is False.""".strip(),
-
-        weights = """
-    weights : array-like or callable or dict, optional
-        if provided, is used instead if individual axes' `weights` attributes
-
-        A weights array can be built from individual axes' weights, either
-        as a parameter to this function, or as a permanent axis attribute
-        defined for the relevant axes. Weights can be of the form:
-
-        - 1-D array-like : for 1-D arrays or if the transformation
-        is to be applied to one axis only (via `axis` parameter)
-
-        - callable : like above, will be applied on the axis specified 
-        by the `axis` parameter, if provided
-
-        If passed as a parameter to the weighted transformation, weights
-        can also be provided as a dictionary. The keys must be axis names 
-        or integer ranks, and values one of the accepted types.""".strip(),
-
-        note = """
-    Notes
-    -----
-    The weights actually used in the transformation can be checked 
-    via the DimArray._get_weights() method (experimental)""".strip(),
-
-        descr = """
-    This transformation can be weighted if a non-None `weights` parameter is provided, or if
-    one of the axes has a non-None `weights` attribute. Otherwise, a standard, unweighted 
-    transformation is performed.""".strip(),
-
-)
-
-#
-# Return a weight for each array element, used for `mean`, `std` and `var`
-#
-@format_doc(**_doc_weights)
-def _get_weights(self, axis=None, mirror_nans=True, weights=None):
-    """ Build array of weights based on individual axes' weights attribute
-
-    Parameters
-    ----------
-    axis : int or str, optional
-        if None a N-D weight is created, otherwise 1-D only
-    mirror_nans : bool, optional
-        mirror `values`'s NaNs in created weights (True by default)
-    {weights}
-
-    Returns
-    -------
-    full_weights : DimArray of weights, or None
-
-    See Also
-    --------
-    DimArray.mean, DimArray.var, DimArray.std
-
-    Examples
-    --------
-
-    Simpler examples are found Dimarray.mean's doc.
-    These are mostly meant as doctest.
-
-    >>> from dimarray import DimArray
-    >>> np.random.seed(0) # to make results reproducible
-    >>> v = DimArray(np.random.rand(3,2), axes=[[-80, 0, 80], [-180, 180]], dims=['lat','lon'])
-    >>> v._get_weights() # None
-    >>> w = lambda x : np.cos(np.radians(v.lat))
-    >>> v.axes['lat'].weights = w
-    >>> v._get_weights()
-    dimarray: 6 non-null elements (0 null)
-    0 / lat (3): -80 to 80
-    1 / lon (2): -180 to 180
-    array([[ 0.17364818,  0.17364818],
-           [ 1.        ,  1.        ],
-           [ 0.17364818,  0.17364818]])
-    >>> v.axes['lat'].weights = None
-    >>> v._get_weights() # None
-
-    Can pass the weigthts as a dictionary 
-
-    >>> v._get_weights(weights={{'lat':w}}) 
-    dimarray: 6 non-null elements (0 null)
-    0 / lat (3): -80 to 80
-    1 / lon (2): -180 to 180
-    array([[ 0.17364818,  0.17364818],
-           [ 1.        ,  1.        ],
-           [ 0.17364818,  0.17364818]])
-
-    Or by indicating the axis
-    
-    >>> v._get_weights(weights=w, axis='lat') 
-    dimarray: 6 non-null elements (0 null)
-    0 / lat (3): -80 to 80
-    1 / lon (2): -180 to 180
-    array([[ 0.17364818,  0.17364818],
-           [ 1.        ,  1.        ],
-           [ 0.17364818,  0.17364818]])
-
-    # It possible to mix up axis and dict-input weights
-    #
-    >>> v.axes['lat'].weights = w
-
-    # >>> v._get_weights(weights={{'lon':lambda x:((x+180)/100)**2}}) 
-    # dimarray: 6 non-null elements (0 null)
-    # 0 / lat (3): -80 to 80
-    # 1 / lon (2): -180 to 180
-    # array([[ 0.       ,  1.5628336],
-    #        [ 0.       ,  9.       ],
-    #        [ 0.       ,  1.5628336]])
-
-    But if the transformation is required on one axis only, 
-    weights on other axes are not accounted for since they
-    would not affect the result of the transformation.
-
-    >>> v._get_weights(weights={{'lon':lambda x:((x+180)/100)**2}}, axis='lat') 
-    dimarray: 6 non-null elements (0 null)
-    0 / lat (3): -80 to 80
-    1 / lon (2): -180 to 180
-    array([[ 0.17364818,  0.17364818],
-           [ 1.        ,  1.        ],
-           [ 0.17364818,  0.17364818]])
-
-    What is provided as argument overrides axis attribute. 
-    Here when the weights are provided for one axis only:
-
-    >>> v._get_weights(weights=lambda x:((x+180)/100)**2, axis='lat') 
-    dimarray: 6 non-null elements (0 null)
-    0 / lat (3): -80 to 80
-    1 / lon (2): -180 to 180
-    array([[ 1.,  1.],
-           [ 1.,  1.],
-           [ 4.,  4.]])
-
-    Or when the weight is provided as a numpy array
-
-    >>> w = np.array([[ 1.,  1.],
-    ...               [ 1.,  1.],
-    ...               [ 4.,  4.]])
-    >>> v._get_weights(weights=w)
-    dimarray: 6 non-null elements (0 null)
-    0 / lat (3): -80 to 80
-    1 / lon (2): -180 to 180
-    array([[ 1.,  1.],
-           [ 1.,  1.],
-           [ 4.,  4.]])
-
-    Or even as DimArray (which will be broadcast with the appropriate shape)
-
-    >>> wa = DimArray(w[:,0], axes=[v.axes['lat']])
-    >>> v._get_weights(weights=wa)
-    dimarray: 6 non-null elements (0 null)
-    0 / lat (3): -80 to 80
-    1 / lon (2): -180 to 180
-    array([[ 1.,  1.],
-           [ 1.,  1.],
-           [ 4.,  4.]])
-
-    Any nans in the data will induce 0 in the weights, 
-    since the element will then be discarded or the result be NaN anyway.
-
-    >>> v.values[1, 0] = np.nan
-    >>> v._get_weights(weights=w)
-    dimarray: 6 non-null elements (0 null)
-    0 / lat (3): -80 to 80
-    1 / lon (2): -180 to 180
-    array([[ 1.,  1.],
-           [ 0.,  1.],
-           [ 4.,  4.]])
-    """
-    # avoid circular import
-    from dimarray import DimArray
-
-    # dims: axes from which the weights should be computed
-    if axis is None:
-        dims = self.dims
-    elif type(axis) is tuple:
-        dims = axis
-    else:
-        dims = (axis,)
-
-    # common case: no weight is provided and no Axis has weights
-    # if is enough to check on all axes to be reduced, weights on other axes 
-    # make no difference to the reduction transformation
-    if weights is None and np.all([self.axes[dim].weights is None for dim in dims]):
-        return None
-
-    # for 1-D array, same as axis is None is like axis=0
-    if axis is None and self.ndim == 1:
-        axis = 0
-
-    # check `weights` type
-
-    # special case: if weights is DimArray, just use it as full weight
-    if isinstance(weights, DimArray):
-        full_weights = weights.broadcast(self.axes) # broadcast weights on appropriate dimension
-
-    # special case II: numpy array of appropriate dimension ==> just use it as full weight
-    elif isinstance(weights, np.ndarray) and weights.ndim > 1:
-        assert weights.shape == self.shape, "`weights` must be either 1-D or have same shape as DimArray"
-        full_weights = DimArray(weights, self.axes)
-
-    # make axis weights based by combining single-axis weights (standard form)
-    else:
-        array_type = isinstance(weights, np.ndarray) and weights.dim == 1
-        dict_type = isinstance(weights, dict)
-        assert weights is None or callable(weights) or array_type or dict_type, "unexpected type for `weights`"
-        assert weights is None or not (not dict_type and axis is None), "`weights` type incompatible with flattening operation, provide `axis` or use dict-type weights"
-        assert weights is None or not (not dict_type and type(axis) is tuple), "`weights` type incompatible with flattening operation, use dict-type weights"
-
-        # make weights dict-like  (operation on single axis)
-        if not dict_type: 
-            weights = {axis: weights}
-
-        # Create weights from the axes (only compute for the axes to be reduced)
-        full_weights = 1.
-        for axis in dims:
-            ax = self.axes[axis]
-
-            # get proper weights array: from `Axis` or user-provided on-the-fly
-            if weights is not None and axis in weights.keys():
-                axis_weights = weights[axis]
-            else:
-                axis_weights = ax.weights
-
-            # if weight is None, just skip it
-            if axis_weights is None: 
-                continue
-
-            # broadcast and multiply weights for all axes involved 
-            full_weights = ax._get_weights(axis_weights).reshape(dims) * full_weights
-
-        # SPECIAL CASE: NO weights (e.g. weights parameter provided as {} or {'lat':None} and no axis weights attributes):
-        if full_weights is 1.:
-            return None
-
-        full_weights = full_weights.broadcast(self.axes)
-
-    # set 0 in a where there are NaNs. 
-    if mirror_nans and anynan(self.values):
-        full_weights.values[np.isnan(self.values)] = 0
-
-    assert full_weights is not None
-    
-    return full_weights
-
-
-@format_doc(**_doc_weights)
-def mean(self, axis=None, skipna=False, weights=None):
-    """ mean over an axis or sequence of axes, possibly weighted 
-
-    {descr} 
-
-    Parameters
-    ----------
-    {axis}
-    {skipna}
-    {weights}
-    
-    Returns
-    -------
-    DimArray instance or scalar, consistently with ndarray behaviour
-
-    {note}
-
-    See Also
-    --------
-    DimArray.var, DimArray.std, DimArray._get_weights
-
-    Examples
-    --------
-    >>> from dimarray import DimArray
-    >>> np.random.seed(0) # to make results reproducible
-    >>> v = DimArray(np.random.rand(3,2), axes=[[-80, 0, 80], [-180, 180]], dims=['lat','lon'])
-
-    Classical, unweighted mean:
-
-    >>> v.mean() 
-    0.58019972362897432
-
-    Weighted mean
-
-    >>> w = np.cos(np.radians(v.lat))
-    >>> v.mean(weights={{'lat':w}})  # only lat axis is weighted
-    0.57628879031663871
-
-    Make the change permanent
-
-    >>> v.axes['lat'].weights = w
-    >>> v.mean()
-    0.57628879031663871
-
-    Check the weights being used (experimental)
-
-    >>> v._get_weights()
-    dimarray: 6 non-null elements (0 null)
-    0 / lat (3): -80 to 80
-    1 / lon (2): -180 to 180
-    array([[ 0.17364818,  0.17364818],
-           [ 1.        ,  1.        ],
-           [ 0.17364818,  0.17364818]])
-    """
-
-    # Proceed to a weighted mean
-    weights = self._get_weights(weights=weights, axis=axis, mirror_nans=skipna)
-
-    # if no weights, just apply numpy's mean
-    if weights is None:
-        return apply_along_axis(self, "mean", axis=axis, skipna=skipna)
-
-    # weighted mean
-    sum_values = apply_along_axis(self*weights, "sum", axis=axis, skipna=skipna)
-    sum_weights = apply_along_axis(weights, "sum", axis=axis, skipna=skipna)
-    return sum_values / (sum_weights + 0.)
-
-@format_doc(**_doc_weights)
-def var(self, axis=None, skipna=False, weights=None, ddof=0):
-    """ variance over an axis or sequence of axes, possibly weighted 
-
-    Parameters
-    ----------
-    {axis}
-    {skipna}
-    {weights}
-    ddof : int, optional
-            "Delta Degrees of Freedom": the divisor used in the calculation is
-             ``N - ddof``, where ``N`` represents the number of elements. By default `ddof` is zero.
-            Note ddof is ignored when weights are used
-            Default is 0.
-    
-    Returns
-    -------
-    DimArray instance or scalar, consistently with ndarray behaviour
-
-    {note}
-
-    See Also
-    --------
-    DimArray.mean, DimArray.std, DimArray._get_weights
-    """
-    # Proceed to a weighted var
-    weights = self._get_weights(weights=weights, axis=axis, mirror_nans=skipna)
-
-    # if no weights, just apply numpy's var
-    if weights is None:
-        return apply_along_axis(self, "var", axis=axis, skipna=skipna, ddof=ddof)
-
-    # weighted mean
-    mean = self.mean(axis=axis, skipna=skipna, weights=weights)
-    dev = (self-mean)**2
-    return dev.mean(axis=axis, skipna=skipna, weights=weights)
-
-# add standard deviation
-def std(self, *args, **kwargs):
-    return self.var(*args, **kwargs)**0.5
-
-std.__doc__ = var.__doc__.replace('variance','standard deviation').replace('var','std').replace('DimArray.std','DimArray.var') # last one for See Also
-
 #
 # linear interpolation along an axis
 #
@@ -1060,7 +689,7 @@ def interp_axis(self, values, axis=0, left=np.nan, right=np.nan, issorted=None):
     >>> a.interp_axis([1,2,3])
     dimarray: 3 non-null elements (0 null)
     0 / x0 (3): 1 to 3
-    array([ 3. ,  3.5,  4. ])
+    array([3. , 3.5, 4. ])
     
     Axis is not sorted
 
@@ -1068,7 +697,7 @@ def interp_axis(self, values, axis=0, left=np.nan, right=np.nan, issorted=None):
     >>> a.interp_axis([1,2,3])
     dimarray: 3 non-null elements (0 null)
     0 / x0 (3): 1 to 3
-    array([ 1.,  2.,  3.])
+    array([1., 2., 3.])
 
     N-Dimensional
 
@@ -1077,8 +706,8 @@ def interp_axis(self, values, axis=0, left=np.nan, right=np.nan, issorted=None):
     dimarray: 6 non-null elements (0 null)
     0 / x0 (2): 'a' to 'b'
     1 / x1 (3): 0.5 to 2.0
-    array([[ 1.5,  2.5,  3. ],
-           [ 4.5,  5.5,  6. ]])
+    array([[1.5, 2.5, 3. ],
+           [4.5, 5.5, 6. ]])
 
     Out-of-bound handling (nan by default, but can be changed via left, right)
 
@@ -1123,7 +752,7 @@ def interp_like(self, other, **kwargs):
     >>> a.interp_like(b)
     dimarray: 3 non-null elements (0 null)
     0 / x1 (3): 1 to 3
-    array([ 3. ,  3.5,  4. ])
+    array([3. , 3.5, 4. ])
     """
     if hasattr(other, 'axes'):
         axes = other.axes
