@@ -1,13 +1,12 @@
 """ collection of base obeje
 """
-from __future__ import absolute_import
-from collections import OrderedDict as odict
 import warnings, copy
 import numpy as np
 
 import dimarray as da  # for the doctest, so that they are testable via py.test
-from dimarray.tools import format_doc
+from dimarray.tools import format_doc, isscalar
 from dimarray.config import get_option
+from dimarray.compat.pycompat import dictkeys, dictvalues
 
 from .core import DimArray, array, Axis, Axes
 from .core import align as align_axes, stack, concatenate
@@ -15,7 +14,7 @@ from .core.align import _check_stack_args, _get_axes, stack, concatenate, _check
 from .core.transform import interp_like, _interp_internal_from_weight, _interp_internal_get_weights, _interp_internal_maybe_sort
 from .core import pandas_obj
 from .core.bases import AbstractDataset, GetSetDelAttrMixin, OpMixin
-from .core.prettyprinting import repr_dataset
+from .prettyprinting import repr_dataset
 
 class DatasetAxes(Axes):
     """Dataset axes, overloaded to propagate modifications to the individual arrays
@@ -24,6 +23,7 @@ class DatasetAxes(Axes):
         super(DatasetAxes, self).__init__()
         assert isinstance(ds, Dataset), "DatasetAxes can only be initialized empty with a Dataset"
         self._ds = ds  # attached dataset
+
     def __setitem__(self, key, item):
         super(DatasetAxes, self).__setitem__(key, item)
         # also apply the change to the contained DimArrays
@@ -33,8 +33,15 @@ class DatasetAxes(Axes):
                 continue
             dima.axes[key] = self[key]
 
-class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
-# class Dataset(AbstractDataset, odict):
+    def __deepcopy__(self, memo):
+        ' deepcopy interface otherwise fails '
+        new = type(self)(self._ds)
+        for ax in self:
+            list.append(new, copy.deepcopy(ax))
+        return new
+
+
+class Dataset(AbstractDataset, dict, OpMixin, GetSetDelAttrMixin):
     """ Container for a set of aligned objects
     
     >>> ds = da.Dataset()
@@ -57,20 +64,21 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
         """
         assert not {'axes','keys'}.issubset(kwargs.keys()) # just to check bugs due to back-compat ==> TO BE REMOVED AFTER DEBUGGING
 
-        # check input arguments: same init as odict
-        data = odict(*args, **kwargs)
+        # check input arguments: same init as dict
+        data = dict(*args)
+        for k in sorted(kwargs):
+            data[k] = kwargs[k]  # sorted keys for consistent output
 
         # Basic initialization
         #self._axes = Axes()
         self._axes = DatasetAxes(self)
-        self._attrs = odict()
+        self._attrs = dict()
 
         # initialize an ordered dictionary
         super(Dataset, self).__init__()
-        #self.data = odict()
 
-        values = data.values()
-        keys = data.keys()
+        values = dictvalues(data)
+        keys = dictkeys(data)
 
         # Check everything is a DimArray
         #for key, value in zip(keys, values):
@@ -79,7 +87,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
                 values[i] = self._constructor(values[i])
 
         # Align objects
-        values = align_axes(*values)
+        values = align_axes(values)
 
         # Append object (will automatically update self.axes)
         for key, value in zip(keys, values):
@@ -218,7 +226,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
             self._maybe_delete_axes(_maybe_obsolete_axes)
 
     def copy(self):
-        ds2 = super(Dataset, self).copy() # odict method, copy axes but not metadata
+        ds2 = Dataset({k : v for k, v in self.items()})
         ds2.attrs.update(self.attrs)
         return ds2
 
@@ -293,7 +301,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
             keys = self.keys()
 
         # align all variables to the same dimensions
-        data = odict()
+        data = dict()
 
         for k in keys:
             data[k] = self[k].reshape(self.dims).broadcast(self.axes)
@@ -386,7 +394,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
         # automatically read all variables to load (except for the dimensions)
         if names is None:
             names = self.keys()
-        elif isinstance(names, basestring):
+        elif isinstance(names, str):
             raise TypeError("Please provide a sequence of variables to read.")
 
         tuple_indices = self._get_indices(indices, axis=axis, tol=tol, keepdims=keepdims, indexing=indexing)
@@ -409,7 +417,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
         if axis is not None: axis = self.axes[axis].name
         kwargs['axis'] = axis
 
-        d = self.to_odict()
+        d = self.to_dict()
         for k in self.keys():
             if axis is not None and axis not in self[k].dims: 
                 continue
@@ -425,7 +433,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
         --------
         >>> a = DimArray([1,2,3], axes=('time', [1950, 1951, 1952]))
         >>> b = DimArray([[11,22,33],[44,55,66]], axes=[('items',['a','b']), ('time', [1950, 1951, 1952])])
-        >>> ds = Dataset(a=a, b=b)
+        >>> ds = Dataset(([('a',a),('b',b)]))  # keep order for doctest
         >>> ds.mean(axis='time')
         Dataset of 2 variables
         0 / items (2): 'a' to 'b'
@@ -452,7 +460,8 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
     def to_odict(self):
         """ export to ordered dict
         """
-        return odict([(nm, self[nm]) for nm in self.keys()])
+        warnings.warn("Deprecated. Use to_dict", FutureWarning)
+        return self.to_dict()
 
     def set_axis(self, values=None, axis=0, name=None, inplace=True, **kwargs):
         """ Set axis values, name and attributes of the Dataset
@@ -476,8 +485,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
         **kwargs : key-word arguments
             Also reset other axis attributes, which can be single metadata
             or other axis attributes, via using `setattr`
-            This includes special attributes `weights` and `attrs` (the latter
-            reset all attributes)
+            This includes special attribute `attrs` (which reset all attributes)
 
         Returns
         -------
@@ -552,7 +560,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
             ds = self.copy()
 
         if isinstance(mapper, dict):
-            iterkeys = mapper.iteritems()
+            iterkeys = mapper.items()
         else:
             if not callable(mapper):
                 raise TypeError("mapper must be callable")
@@ -576,7 +584,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
             ds = self.copy()
 
         if isinstance(mapper, dict):
-            iterkeys = mapper.iteritems()
+            iterkeys = mapper.items()
         else:
             if not callable(mapper):
                 raise TypeError("mapper must be callable")
@@ -660,7 +668,7 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
             newaxis = values
             values = newaxis.values
             axis = newaxis.name
-        elif np.isscalar(values) or type(values) is slice:
+        elif isscalar(values) or type(values) is slice:
             raise TypeError("Please provide list, array-like or Axis object to perform re-indexing")
         else:
             values = np.asarray(values)
@@ -747,17 +755,17 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
         dimarray: 4 non-null elements (0 null)
         0 / x0 (2): 0 to 1
         1 / x1 (2): 0 to 1
-        array([[ 1.,  1.],
-               [ 1.,  1.]])
+        array([[1., 1.],
+               [1., 1.]])
         >>> ds += 1
         >>> ds['b']
         dimarray: 4 non-null elements (0 null)
         0 / x0 (2): 0 to 1
         1 / x1 (2): 0 to 1
-        array([[ 1.,  2.],
-               [ 2.,  3.]])
+        array([[1., 2.],
+               [2., 3.]])
         """
-        assert isinstance(other, Dataset) or np.isscalar(other), "can only combine Datasets objects (func={})".format(func.__name__)
+        assert isinstance(other, Dataset) or isscalar(other), "can only combine Datasets objects (func={})".format(func.__name__)
         # align all axes first
         reindex = get_option("op.reindex")
         if reindex and hasattr(other, 'axes') and other.axes != self.axes:
@@ -779,7 +787,8 @@ class Dataset(AbstractDataset, odict, OpMixin, GetSetDelAttrMixin):
             res[k] = self[k]._unary_op(func)
         return res
 
-def stack_ds(datasets, axis, keys=None, align=False):
+
+def stack_ds(datasets, axis, keys=None, align=False, **kwargs):
     """ stack dataset along a new dimension
 
     Parameters
@@ -787,7 +796,8 @@ def stack_ds(datasets, axis, keys=None, align=False):
     datasets: sequence or dict of datasets
     axis: str, new dimension along which to stack the dataset 
     keys, optional: stack axis values, useful if dataset is a sequence, or a non-ordered dictionary
-    align, optional: if True, align axes (via reindexing) prior to stacking
+    align, optional: if True, align axes (via reindexing) *prior* to stacking
+    **kwargs : optional key-word arguments passed to align, if align is True
 
     Returns
     -------
@@ -795,14 +805,14 @@ def stack_ds(datasets, axis, keys=None, align=False):
 
     See Also
     --------
-    concatenate_ds
+    concatenate_ds, stack, sort_axis
 
     Examples
     --------
     >>> a = DimArray([1,2,3], dims=('dima',))
     >>> b = DimArray([11,22], dims=('dimb',))
-    >>> ds = Dataset({'a':a,'b':b}) # dataset of 2 variables from an experiment
-    >>> ds2 = Dataset({'a':a*2,'b':b*2}) # dataset of 2 variables from a second experiment
+    >>> ds = Dataset(a=a,b=b) # dataset of 2 variables from an experiment
+    >>> ds2 = Dataset(a=a*2,b=b*2) # dataset of 2 variables from a second experiment
     >>> stack_ds([ds, ds2], axis='stackdim', keys=['exp1','exp2'])
     Dataset of 2 variables
     0 / stackdim (2): 'exp1' to 'exp2'
@@ -811,8 +821,6 @@ def stack_ds(datasets, axis, keys=None, align=False):
     a: ('stackdim', 'dima')
     b: ('stackdim', 'dimb')
     """
-    #if not isinstance(axis, str): raise TypeError("axis parameter must be str")
-
     # make a sequence of datasets
     datasets, keys = _check_stack_args(datasets, keys) 
 
@@ -820,6 +828,9 @@ def stack_ds(datasets, axis, keys=None, align=False):
     dims = _get_dims(*datasets)
     axis = _check_stack_axis(axis, dims) 
 
+    if align:
+        datasets = da.align(datasets, strict=True, **kwargs)
+
     # find the list of variables common to all datasets
     variables = None
     for ds in datasets:
@@ -836,19 +847,21 @@ def stack_ds(datasets, axis, keys=None, align=False):
     dataset = Dataset()
     for v in variables:
         arrays = [ds[v] for ds in datasets]
-        array = stack(arrays, axis=axis, keys=keys, align=align)
+        array = stack(arrays, axis=axis, keys=keys, align=False)
         dataset[v] = array
 
     return dataset
 
 
-def concatenate_ds(datasets, axis=0):
+def concatenate_ds(datasets, axis=0, align=False, **kwargs):
     """ concatenate two datasets along an existing dimension
 
     Parameters
     ----------
     datasets: sequence of datasets 
     axis: axis along which to concatenate
+    align, optional: if True, align secondary axes (via reindexing) prior to concatenating
+    **kwargs : optional key-word arguments passed to align, if align is True
 
     Returns
     -------
@@ -858,16 +871,16 @@ def concatenate_ds(datasets, axis=0):
 
     See Also
     --------
-    stack_ds
+    stack_ds, concatenate, sort_axis
 
     Examples
     --------
     >>> a = da.zeros(axes=[list('abc')], dims=('x0',))  # 1-D DimArray
     >>> b = da.zeros(axes=[list('abc'), [1,2]], dims=('x0','x1')) # 2-D DimArray
-    >>> ds = Dataset({'a':a,'b':b}) # dataset of 2 variables from an experiment
+    >>> ds = Dataset(a=a,b=b) # dataset of 2 variables from an experiment
     >>> a2 = da.ones(axes=[list('def')], dims=('x0',)) 
     >>> b2 = da.ones(axes=[list('def'), [1,2]], dims=('x0','x1')) # 2-D DimArray
-    >>> ds2 = Dataset({'a':a2,'b':b2}) # dataset of 2 variables from a second experiment
+    >>> ds2 = Dataset(a=a2,b=b2) # dataset of 2 variables from a second experiment
     >>> concatenate_ds([ds, ds2])
     Dataset of 2 variables
     0 / x0 (6): 'a' to 'f'
@@ -885,11 +898,18 @@ def concatenate_ds(datasets, axis=0):
         else:
             assert sorted(ds.keys()) == sorted(variables), "variables differ across datasets"
 
+    if align:
+        # all dataset axes
+        axis_nm = datasets[0].axes[axis].name
+        aligned_dims = [d for d in _get_dims(*datasets) if d != axis_nm]
+        for d in aligned_dims:
+            datasets = da.align(datasets, axis=d, strict=True, **kwargs)
+
     # Compute concatenated dataset
     dataset = Dataset()
     for v in variables:
         arrays = [ds[v] for ds in datasets]
-        array = concatenate(arrays, axis=axis)
+        array = concatenate(arrays, axis=axis, align=False, _no_check=align)
         dataset[v] = array
 
     return dataset
