@@ -1,9 +1,6 @@
-""" NetCDF I/O to access and save geospatial data
-"""
 import os
 import re
 import glob, copy
-from collections import OrderedDict as odict
 from functools import partial
 import warnings
 import numpy as np
@@ -11,14 +8,13 @@ import netCDF4 as nc
 import dimarray as da
 #from geo.data.index import to_slice, _slice3D
 
-from dimarray.compat.pycompat import basestring, zip
-from dimarray.tools import format_doc
+from dimarray.tools import format_doc, isscalar
 from dimarray.dataset import Dataset, concatenate_ds, stack_ds
 from dimarray.core import DimArray, Axis, Axes
 from dimarray.config import get_option
 from dimarray.core.bases import AbstractDimArray, AbstractDataset, AbstractAxis, GetSetDelAttrMixin, AbstractAxes
 # from dimarray.core.metadata import _repr_metadata
-from dimarray.core.prettyprinting import repr_axis, repr_axes, repr_dimarray, repr_dataset, repr_attrs
+from dimarray.prettyprinting import repr_axis, repr_axes, repr_dimarray, repr_dataset, repr_attrs
 
 from .conventions import encode_cf_datetime, decode_cf_datetime
 
@@ -83,7 +79,7 @@ def istimevariable(ncvar):
     """
     # return hastimeunits(ncvar) or \
     return (
-        ncvar.size > 0 and isinstance(ncvar[0], basestring) \
+        ncvar.size > 0 and isinstance(ncvar[0], str) \
          and re.match('\d\d(\d\d)?-\d\d-\d\d',ncvar[0]))
 
 class NCTimeVariableWrapper(object):
@@ -175,7 +171,7 @@ class DatasetOnDisk(GetSetDelAttrMixin, NetCDFOnDisk, AbstractDataset):
             try:
                 self._ds = nc.Dataset(f, mode=mode, clobber=clobber, diskless=diskless, persist=persist, format=format)
             except UserWarning as error:
-                print error
+                print(error)
             except Exception as error: # indicate file name when issuing error
                 raise IOError("{}\n=> failed to open {} (mode={}, clobber={})".format(str(error), f, mode, clobber)) # easier to handle
         # assert self._kwargs is not None
@@ -229,7 +225,7 @@ class DatasetOnDisk(GetSetDelAttrMixin, NetCDFOnDisk, AbstractDataset):
         if names is None:
             dims = self.dims
             names = self.keys()
-        elif isinstance(names, basestring):
+        elif isinstance(names, str):
             return self[names].read(indices=indices, axis=axis, indexing=indexing, tol=tol, keepdims=keepdims)
         else:
             dims = []
@@ -507,7 +503,7 @@ class DimArrayOnDisk(GetSetDelAttrMixin, NetCDFVariable, AbstractDimArray):
                     self.axes[ax.name][idx] = axis
                 else:
                     # dimension variable already written, simple check
-                    ondisk = self.axes[ax.name][idx if not np.isscalar(idx) else [idx]].values
+                    ondisk = self.axes[ax.name][idx if not (np.isscalar(idx) or np.ndim(idx)==0)  else [idx]].values
                     inmemory = axis.values
                     # inmemory, _ = maybe_encode_values(axis.values)
                     if not np.all(ondisk == inmemory):
@@ -591,14 +587,14 @@ class DimArrayOnDisk(GetSetDelAttrMixin, NetCDFVariable, AbstractDimArray):
         axes = []
         for i, ix in enumerate(idx_tuple):
             ax = self.axes[i][ix]
-            if not np.isscalar(ax): # do not include scalar axes
+            if np.ndim(ax) != 0: # do not include scalar axes
                 axes.append(ax)
         return axes
 
 class AxisOnDisk(GetSetDelAttrMixin, NetCDFVariable, AbstractAxis):
     def __init__(self, ds, name):
         self._ds = ds
-        if not isinstance(name, basestring):
+        if not isinstance(name, str):
             raise TypeError("only string names allowed")
         self._name = name
 
@@ -657,10 +653,11 @@ class AxisOnDisk(GetSetDelAttrMixin, NetCDFVariable, AbstractAxis):
 
         if np.isscalar(indices): 
             assert values.size == 1
-            if values.ndim == 0:
+            try:
+                values = np.asarray(values).item()
+            except AttributeError:
                 values = values[()]
-            else:
-                values = values[0]
+
 
         ds.variables[name][indices] = values
 
@@ -675,13 +672,16 @@ class AxisOnDisk(GetSetDelAttrMixin, NetCDFVariable, AbstractAxis):
     def __getitem__(self, indices):
         values = self.values[indices]
 
-        # do not produce an Axis object
-        if np.isscalar(values):
-            return values
 
         if isinstance(values, np.ma.MaskedArray):
             values = values.filled(0)
 
+        # do not produce an Axis object
+        if np.isscalar(values):
+            return values
+        elif np.ndim(values) == 0:
+            return np.asarray(values).item()
+
         ax = Axis(values, self._name)
 
         # add metadata
@@ -728,21 +728,21 @@ class AxesOnDisk(AbstractAxes):
         return len(self._dims)
 
     def __getitem__(self, dim):
-        if not isinstance(dim, basestring):
+        if not isinstance(dim, str):
             dim = self.dims[dim]
         elif dim not in self.dims:
             msg = "{} not found in dimensions (dims={}).".format(dim, self.dims)
-            print """An error ocurred? A new dimension can be created via `axes.append(axis)` syntax, 
+            print("""An error ocurred? A new dimension can be created via `axes.append(axis)` syntax, 
 where axis can be a string (unlimited dimension), an Axis instance, 
 or a tuple (name, values).  See help on `axes.append` for more information. 
 Low-level netCDF4 function is also available as `ds.nc.createDimension` 
-and `ds.nc.createVariable`"""
+and `ds.nc.createVariable`""")
             raise KeyError(msg)
         return AxisOnDisk(self._ds, dim)
 
     def __setitem__(self, dim, ax):
         " modify existing axis and possibly create new associated variable " 
-        if not isinstance(dim, basestring):
+        if not isinstance(dim, str):
             dim = self.dims[dim]
 
         if dim not in self._ds.dimensions.keys():
@@ -771,7 +771,7 @@ and `ds.nc.createVariable`"""
             be provided)
         **kwargs : passed to createVariable (compression parameters)
         """
-        if isinstance(ax, basestring):
+        if isinstance(ax, str):
             self._ds.createDimension(ax, size)
             self._dims += (ax,) 
 
@@ -826,7 +826,7 @@ class AttrsOnDisk(object):
     def values(self):
         return [self.obj.getncattr(k) for k in self.obj.ncattrs()]
     def todict(self):
-        return odict(zip(self.keys(), self.values()))
+        return dict(zip(self.keys(), self.values()))
     def __iter__(self):
         for k in self.keys():
             yield k
@@ -863,11 +863,11 @@ def open_nc(file_name, *args, **kwargs):
     DatasetOnDisk of 6 variables (NETCDF4)
     0 / y1 (113): -3400000.0 to -600000.0
     1 / x1 (61): -800000.0 to 700000.0
-    surfvelmag: (u'y1', u'x1')
-    lat: (u'y1', u'x1')
-    lon: (u'y1', u'x1')
-    surfvely: (u'y1', u'x1')
-    surfvelx: (u'y1', u'x1')
+    surfvelmag: ('y1', 'x1')
+    lat: ('y1', 'x1')
+    lon: ('y1', 'x1')
+    surfvely: ('y1', 'x1')
+    surfvelx: ('y1', 'x1')
     mapping: nan
 
     Load variables with [:] syntax, like netCDF4 package
@@ -883,13 +883,15 @@ def open_nc(file_name, *args, **kwargs):
     >>> ds['surfvelmag'].ix[:10, -1] # load first 10 y1 values, and last x1 value
     dimarray: 10 non-null elements (0 null)
     0 / y1 (10): -3400000.0 to -3175000.0
-    array([ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.], dtype=float32)
+    array([0., 0., 0., 0., 0., 0., 0., 0., 0., 0.], dtype=float32)
 
     >>> ds['surfvelmag'].sel(x1=700000, y1=-3400000)
-    0.0
+    dimarray: 1 non-null elements (0 null)
+    array(0., dtype=float32)
 
     >>> ds['surfvelmag'].isel(x1=-1, y1=0)
-    0.0
+    dimarray: 1 non-null elements (0 null)
+    array(0., dtype=float32)
 
     Need to close the Dataset at the end
 
@@ -903,11 +905,11 @@ def open_nc(file_name, *args, **kwargs):
     Dataset of 6 variables
     0 / y1 (113): -3400000.0 to -600000.0
     1 / x1 (61): -800000.0 to 700000.0
-    surfvelmag: (u'y1', u'x1')
-    lat: (u'y1', u'x1')
-    lon: (u'y1', u'x1')
-    surfvely: (u'y1', u'x1')
-    surfvelx: (u'y1', u'x1')
+    surfvelmag: ('y1', 'x1')
+    lat: ('y1', 'x1')
+    lon: ('y1', 'x1')
+    surfvely: ('y1', 'x1')
+    surfvelx: ('y1', 'x1')
     mapping: nan
     """
     return DatasetOnDisk(file_name, *args, **kwargs)
@@ -934,13 +936,6 @@ def read_nc(f, names=None, *args, **kwargs):
         For example, scalar items will be left unchanged whatever indices
         are provided.
 
-    axis : 
-        When reading multiple files and align==True :
-            axis along which to join the dimarrays or datasets (if align is True)
-        When reading one file (deprecated: use {name:values} notation instead):
-            if specified and indices is a slice, scalar or an array, assumes 
-            indexing is along this axis.
-
     indexing : {'label', 'position'}, optional
         Indexing mode. 
         - "label": indexing on axis labels (default)
@@ -954,23 +949,32 @@ def read_nc(f, names=None, *args, **kwargs):
     keepdims : bool, optional 
         keep singleton dimensions (default False)
 
-    align : bool, optional
-        if names is a list of files or a regular expression, pass align=True
-        if the arrays from the various files have to be aligned prior to 
-        concatenation. Similar to dimarray.stack and dimarray.stack_ds
-
-        Only when reading multiple files
-
     axis : str, optional
-        axis along which to join the dimarrays or datasets (if align is True)
-
-        Only when reading multiple files and align==True
+        When reading multiple files, axis along which to join
+        the dimarrays or datasets. It the axis already exist, 
+        the resulting arrays will be concatenated, otherwise
+        they will be stacked along a new array (in the sense 
+        of the numpy functions `concatenate` and `stack`)
 
     keys : sequence, optional
-        new axis values. If not provided, file names will be taken instead.
-        It is always possible to use set_axis later.
+        When reading multiple files, keys for the join axis. If the 
+        axis already exists in the dataset, the concatenated dataset/dimarray
+        will be re-indexed along the provided key, otherwise the keys
+        will be used to create a new axis for stacking. In the latter case,
+        keys' length needs to exactly match the number of input files, and 
+        if not provided, file names will be taken instead. Note you may
+        manually rename the axes later, or use the `set_axis` method.
 
-        Only when reading multiple files and align==True
+    align : bool, optional
+        When reading multiple files, passed to `stack` (new axis) or 
+        `concatenate` (existing axis) to reindex all arrays onto common axes.
+        (in `concatenate` mode, the concatenation axis is *not* re-indexed of course, 
+        only the secondary axes)
+        Default to False.
+
+    **kwargs : optional key-word arguments passed to align, if align is True
+        When reading multiple files, passed to `stack` (new axis) or 
+        This includes: `sort` (False by default) and `join` ('outer' by default)
 
     Returns
     -------
@@ -979,7 +983,7 @@ def read_nc(f, names=None, *args, **kwargs):
 
     See Also
     --------
-    DatasetOnDisk.read, stack, concatenate, stack_ds, concatenate_ds,
+    DatasetOnDisk.read, stack, concatenate, stack_ds, concatenate_ds, align,
     DimArray.write_nc, Dataset.write_nc
 
     Examples
@@ -995,9 +999,9 @@ def read_nc(f, names=None, *args, **kwargs):
     >>> data
     Dataset of 2 variables
     0 / time (451): 1850 to 2300
-    1 / scenario (5): u'historical' to u'rcp85'
-    tsl: (u'time', u'scenario')
-    temp: (u'time', u'scenario')
+    1 / scenario (5): 'historical' to 'rcp85'
+    tsl: ('time', 'scenario')
+    temp: ('time', 'scenario')
     >>> data = read_nc(ncfile,'temp') # only one variable
     >>> data = read_nc(ncfile,'temp', indices={"time":slice(2000,2100), "scenario":"rcp45"})  # load only a chunck of the data
     >>> data = read_nc(ncfile,'temp', indices={"time":1950.3}, tol=0.5)  #  approximate matching, adjust tolerance
@@ -1021,7 +1025,7 @@ def read_nc(f, names=None, *args, **kwargs):
     dimarray: 9114 non-null elements (6671 null)
     0 / model (7): 'CSIRO-Mk3-6-0' to 'MPI-ESM-MR'
     1 / time (451): 1850 to 2300
-    2 / scenario (5): u'historical' to u'rcp85'
+    2 / scenario (5): 'historical' to 'rcp85'
     array(...)
     
     This works on datasets as well:
@@ -1032,9 +1036,9 @@ def read_nc(f, names=None, *args, **kwargs):
     Dataset of 2 variables
     0 / model (7): 'CSIRO-Mk3-6-0' to 'MPI-ESM-MR'
     1 / time (451): 1850 to 2300
-    2 / scenario (5): u'historical' to u'rcp85'
-    tsl: ('model', u'time', u'scenario')
-    temp: ('model', u'time', u'scenario')
+    2 / scenario (5): 'historical' to 'rcp85'
+    tsl: ('model', 'time', 'scenario')
+    temp: ('model', 'time', 'scenario')
     """
     # check for regular expression
     if type(f) is str:
@@ -1072,7 +1076,7 @@ def read_nc(f, names=None, *args, **kwargs):
 
     return obj
 
-def _read_multinc(fnames, names=None, axis=None, keys=None, align=False, concatenate_only=False, **kwargs):
+def _read_multinc(fnames, names=None, axis=None, keys=None, align=False, sort=False, join='outer', concatenate_only=False, **kwargs):
     """ read multiple netCDF files 
 
     Parameters
@@ -1085,7 +1089,11 @@ def _read_multinc(fnames, names=None, axis=None, keys=None, align=False, concate
     keys : sequence, optional
         to be passed to stack_nc, if axis is not part of the dataset
     align : `bool`, optional
-        if True, reindex axis prior to stacking (default to False)
+        if True, reindex axis prior to stacking / concatenating (default to False)
+    sort : `bool`, optional
+        if True, sort common axes prior to stacking / concatenating (defaut to False)
+    join : `str`, optional
+        join method in align (default to 'outer')
     concatenate_only : `bool`, optional
         if True, only concatenate along existing axis (and raise error if axis not existing) 
 
@@ -1140,14 +1148,18 @@ def _read_multinc(fnames, names=None, axis=None, keys=None, align=False, concate
 
     # Join dataset
     if axis in dimensions:
-        if keys is not None: warnings.warn('keys argument will be ignored.', RuntimeWarning)
-        ds = concatenate_ds(datasets, axis=axis)
+        ds = concatenate_ds(datasets, axis=axis, align=align, sort=sort, join=join)
+        if keys is not None: 
+            ds = ds.reindex_axis(keys, axis=axis)
+        # elif sort:
+        #     ds = ds.sort_axis(axis=axis)
+            #warnings.warn('keys argument will be ignored.', RuntimeWarning)
 
     else:
         # use file name as keys by default
         if keys is None:
             keys = [os.path.splitext(fn)[0] for fn in fnames]
-        ds = stack_ds(datasets, axis=axis, keys=keys, align=align)
+        ds = stack_ds(datasets, axis=axis, keys=keys, align=align, sort=sort, join=join)
 
     return ds
 
@@ -1163,7 +1175,7 @@ def summary_nc(fname, name=None, metadata=False):
     with DatasetOnDisk(fname) as obj:
         if name is not None:
             obj = obj[name]
-        print(obj.__repr__(metadata=metadata))
+        print((obj.__repr__(metadata=metadata)))
 
 
 def _maybe_open_file(f, mode='r', clobber=None, verbose=False, format=None):
@@ -1214,19 +1226,19 @@ def _maybe_open_file(f, mode='r', clobber=None, verbose=False, format=None):
 
         try:
             f = nc.Dataset(fname, mode, clobber=clobber, format=format)
-        except UserWarning, msg:
-            print msg
-        except Exception, msg: # raise a weird RuntimeError
+        except UserWarning as msg:
+            print(msg)
+        except Exception as msg: # raise a weird RuntimeError
             #print "read from",fname
             raise IOError("{} => failed to opend {} in mode {}".format(msg, fname, mode)) # easier to handle
 
         if verbose: 
             if 'r' in mode:
-                print "read from",fname
+                print("read from",fname)
             elif 'a' in mode:
-                print "append to",fname
+                print("append to",fname)
             else:
-                print "write to",fname
+                print("write to",fname)
         close = True
     else:
         close = False # leave it open
