@@ -5,6 +5,8 @@ from numpy.testing import assert_equal, assert_almost_equal
 import dimarray as da
 from dimarray.tools import anynan
 
+string_types = (str,)
+
 SEED = None
 #
 # Create numpy arrays of various types
@@ -31,13 +33,13 @@ def create_array_axis(size, dtype, regular=True, seed=SEED):
         values = np.random.shuffle(values)
     return np.asarray(values, dtype=dtype)
 
-def create_metadata(types=[str, unicode, int, float, list]):
+def create_metadata(types=(int, float, list)+string_types):
     """ return a dictionary of metadata with various types
     """
     meta = {}
     letters = list('abcdefghijklmnopqrstuvwxyz')
     for i, t in enumerate(types):
-        if t in (str, unicode): 
+        if t in string_types: 
             val = t('some string')
         elif t in (list, np.ndarray):
             val = t([1,2])
@@ -64,9 +66,7 @@ def create_dimarray(shape=(2,3), dtype=float, axis_dtypes=float, dims=None, seed
 
     return da.DimArray(values, axes, dims=dims, **meta)
 
-# def create_dataset(seed=SEED, dtypes = ("float", "int","int32", "int64")):
-def create_dataset(seed=SEED, dtypes = ("float", "int","int32", "int64", "str", "unicode")):
-    # a = create_dimarray((2,3,2,1,1), float, (float, int, str, unicode, object), seed=seed)
+def create_dataset(seed=SEED, dtypes = ("float", "int","int32", "int64")+tuple(t.__name__ for t in string_types)):
     ds = da.Dataset()
     ds['many_axes'] = create_dimarray((2,3,2), float, (float, int, str), seed=seed) # test axis types
     ds['shared_axis'] = ds['many_axes'].ix[0] # shares axes with "many_axes"  # test shared axes
@@ -86,7 +86,6 @@ def assert_equal_axes(actual, expected, metadata=True):
     assert actual == expected
     if metadata:
         assert_equal_metadata(actual.attrs, expected.attrs)
-        assert np.all(actual.weights == expected.weights)
 
 def assert_equal_dimarrays(actual, expected, metadata=True, approx=False):
     assert isinstance(expected, da.DimArray)
@@ -120,9 +119,9 @@ def assert_equal_datasets(actual, expected, metadata=True, approx=False):
         try:
             assert_equal_dimarrays(actual[k], expected[k], approx=approx, metadata=metadata)
         except:
-            print actual[k]
-            print expected[k]
-            print k
+            print(actual[k])
+            print(expected[k])
+            print(k)
             raise
     # check the metadata
     if metadata:
